#!/usr/bin/env bash

# bash xgiga_dump_embedding_w_text.sh mspm4_xgiga en zh zh_CN finetune_pretrainedMspm4_freezeDecoder None /opt/tiger/sumtest/tmpdata/test.x.zh test_x_zh 

DATAVER="$1"
TRAIN_LG="$2"
TEST_LG="$3"
TEST_LG_TAG="$4"
NAME="$5"
EXTEND_DICT="$6"
INPATH="$7"
EMBDIR="$8"

argslist=""
for (( i = 9; i <= $# ; i++ ))
  do
    j=${!i}
    argslist="${argslist} $j "
  done
echo $argslist >&2

tokenize(){   
    base_dir=$(pwd)

    INPUT=$1
    OUTPUT=$2
    DICT=$3

    echo "tokenize ${INPUT} to ${OUTPUT} using mbart's spm..."
    # setup MBART
    MBART=/home/tiger/mbart.cc25
    if [ ! -d $MBART ]; then
        hadoop fs -copyToLocal hdfs://haruna/home/byte_arnold_hl_mlnlc/user/wuxianze.0/Workspace/Multilingual/pretrained/mbart.CC25.tar.gz /home/tiger
        tar -xvzf /home/tiger/mbart.CC25.tar.gz -C /home/tiger
    fi
    MODEL=$MBART/sentence.bpe.model
    python3 ${base_dir}/fairseq/scripts/spm_encode.py --model=$MODEL < $INPUT > $OUTPUT

    echo "first line of ${OUTPUT}"
    head -n 1 $OUTPUT
}

cd "$(dirname $0)" || return
echo "Install fairseq" >&2

sudo mkdir -p /usr/lib/python3.7/site-packages/
sudo pip3 install -e fairseq
pip3 install -r requirements.txt -i http://pypi.byted.org/simple/ --trusted-host=pypi.byted.org

export PYROUGE_HOME_DIR=$(pwd)/RELEASE-1.5.5
export PYROUGE_TEMP_PATH=/opt/tiger

pyrouge_set_rouge_path $PYROUGE_HOME_DIR
chmod +x $PYROUGE_HOME_DIR/ROUGE-1.5.5.pl

prefix=hdfs://haruna/home/byte_arnold_hl_mlnlc/user/wuxianze.0

dataset_path=${prefix}/Datasets/multilingual/data-bin/${DATAVER}/${TEST_LG}
old_checkpoint_path=${prefix}/Workspace/Multilingual/${TRAIN_LG}/checkpoints/${NAME}
new_checkpoint_path=${prefix}/Workspace/Multilingual/xgiga/${TRAIN_LG}/checkpoints/${NAME}
embedding_dir=${prefix}/Workspace/Multilingual/xgiga/${TEST_LG}/embeddings/${NAME}_${TRAIN_LG}
pretrained_path=${prefix}/Workspace/Multilingual/pretrained

# change checkpoint path
hdfs dfs -mkdir -p $embedding_dir

local_root=~/xgiga_dumpEmb_${NAME}
resource_root=${local_root}/resource
output_path=${local_root}/output
model_path=${local_root}/model
local_embedding_path=${local_root}/embedding/${EMBDIR}
mkdir -p ${resource_root}
mkdir -p ${output_path}
mkdir -p ${model_path}
mkdir -p ${local_embedding_path}

local_dataset_path=${resource_root}/dataset
mkdir -p ${local_dataset_path}
hadoop fs -copyToLocal ${dataset_path}/* ${local_dataset_path}
echo "Download resource from ${dataset_path} to ${local_dataset_path}" >&2

local_checkpoint_path=${output_path}/checkpoint_path
mkdir -p ${local_checkpoint_path}
hadoop fs -copyToLocal ${old_checkpoint_path}/checkpoint_best.pt ${local_checkpoint_path}
if [ -e ${local_checkpoint_path}/checkpoint_last.pt ]; then
  echo "Load checkpoints from ${old_checkpoint_path}/checkpoint_best.pt to ${local_checkpoint_path}" >&2
else
  hadoop fs -copyToLocal ${new_checkpoint_path}/checkpoint_best.pt ${local_checkpoint_path}
  echo "Load checkpoints from ${new_checkpoint_path}/checkpoint_best.pt to ${local_checkpoint_path}" >&2
fi

echo "remove ${local_dataset_path}/dict.*.txt"
rm ${local_dataset_path}/dict.*.txt
hdfs dfs -get ${pretrained_path}/dict_extend.txt ${local_dataset_path}
echo "treat ${local_dataset_path}/dict_extend.txt as ${local_dataset_path}/dict.*.txt"
cp ${local_dataset_path}/dict_extend.txt ${local_dataset_path}/dict.doc.txt
cp ${local_dataset_path}/dict_extend.txt ${local_dataset_path}/dict.sum.txt

if [ ! "${EXTEND_DICT}" == "None" ]; then
    echo "Load extend dictionary from ${pretrained_path}/${EXTEND_DICT}.txt to ${local_dataset_path}" >&2
    hdfs dfs -get ${pretrained_path}/${EXTEND_DICT}.txt ${local_dataset_path}
    echo "write the extended dictionary into ${local_dataset_path}/dict.*.txt"
    cat ${local_dataset_path}/${EXTEND_DICT}.txt >> ${local_dataset_path}/dict.doc.txt
    cat ${local_dataset_path}/${EXTEND_DICT}.txt >> ${local_dataset_path}/dict.sum.txt
    if [ ! -e ${local_dataset_path}/${EXTEND_DICT}.txt ]; then
        echo "[ERROR] Load extend dictionary ${EXTEND_DICT}.txt failed!" >&2
    fi
fi

# # tokenize raw text
tokenize $INPATH $INPATH.spm ${local_dataset_path}/dict.doc.txt
cat $INPATH.spm | sed -e "s/$/ ./" > $INPATH.spm.special 

echo "Finish download files" >&2
langs=ar_AR,cs_CZ,de_DE,en_XX,es_XX,et_EE,fi_FI,fr_XX,gu_IN,hi_IN,it_IT,ja_XX,kk_KZ,ko_KR,lt_LT,lv_LV,my_MM,ne_NP,nl_XX,ro_RO,ru_RU,si_LK,tr_TR,vi_VN,zh_CN

echo "Start dumping embedding..."
suffix=$(echo "$argslist" | sed -e "s/-//g"  -e "s/  */_/g")

python3 fairseq/interactive_dump_embedding.py ${local_dataset_path}  \
    --path ${local_checkpoint_path}/checkpoint_best.pt \
    --task multi_task_from_pretrained_mbart_mspm4 \
    --gen-subset test \
    --source-lang doc --target-lang sum \
    --langs $langs \
    --results-path ${local_embedding_path} \
    --remove-bpe 'sentencepiece'  \
    --truncate-source \
    --prefix-tokens ${TEST_LG_TAG} --doc-lang ${TEST_LG_TAG} \
    --user-dir examples/summarization \
    --batch-size 1 $argslist \
    --input $INPATH.spm.special
 
echo "Put ${local_embedding_path} to ${embedding_dir}" >&2
hadoop fs -put -f ${local_embedding_path}/* ${embedding_dir}/
# sleep 600
